
## Running Polling Example
* Login to Jelastic Dashboard
* Open [examples/polling/index.html](examples/polling/index.html) in text editor and change the following variables
according to current Jelastic Dashboard session:
```
$.jelastic.api.BASE_URL = "{BASE_URL}"; //e.g. https://app.demo.jelastic.com/
$.jelastic.api.SESSION = "{SESSION}"; //current user session id from platform stated above
```
* Open [examples/polling/index.html](examples/polling/index.html) in browser
* Back to Jelastic Dashboard and begin some actions (restart some nodes, install application from Marketplace, etc.)
* See actions progress in [examples/polling/index.html](examples/polling/index.html)  tab

## Library Usage
### ActionPoller
Provides for repetitive polling of the Jelastic server actions.

Options:
* activeInterval - polling interval in milliseconds for active state (has active actions). Default: 1000  (1 second)
* sleepingInterval - polling interval in milliseconds for sleeping state (no active actions). Default: 10000  (10 seconds)
* retriesCount - polls count between active and sleeping state. Default: 3
* reloadCount - polls count between actions history requests for unchanged state. Default: 4

Events:
* **action-started** (trackedAction, data)
* **action-completed** (trackedAction, data)
* **composite-action-started** (compositeActionData, trackedAction)
* **internal-action-completed** (compositeActionData, trackedAction)
* **internal-action-error** (compositeActionData, trackedAction)

Example:

```
var actionPoller = $.jelastic.polling.ActionPoller();
actionPoller.on("action-started", function () { 
    console.log("action started"); 
});
```

For handling such complex actions as JPS installation use composite action handler:  

```
actionPoller.addCompositeActionHandler("installapp", {
    events: [
        "create",
        "deploy",
        "config",
    ],
    
    params: [
        "envName",
        "appName",
        "nodeGroup",
        "jpsType",
        "time",
        "envDisplayName",
        "region",
        "subDomain",
        "isDocker",
        "eventName",
        "appid"
    ],
    
    onStart: function () {}
    onProgress: function () {}
    onError: function () {}
    onSuccess: function () {}
});
```

## How to build your own lib

Clone repository and install libraries:
```
git clone git@git.jelastic.com:sk/jelastic-jquery-lib.git
npm install
```

Build:
```
npm run build
```


## Debugging
```
npm run watch
```