(function ($) {
    $.jelastic = $.jelastic || {};
    $.jelastic.data = $.jelastic.data || {};

    $.jelastic.data.MixedCollection = function (values, options) {
        var self;

        options = $.extend({
            idProperty: "id"
        }, options);

        self = {
            items: [],
            map: {},
            keys: [],
            length: 0,

            add: function (key, object) {
                var old;

                if (arguments.length === 1) {
                    object = arguments[0];
                    key = self.getKey(object);
                }

                if (key !== null && typeof key != 'undefined') {
                    old = self.map[key];

                    if (typeof old != 'undefined') {
                        return self.replace(key, object);
                    }

                    self.map[key] = object;
                }

                self.length += 1;
                self.items.push(object);
                self.keys.push(key);

                return object;
            },

            addAll: function (objects) {
                var args,
                    key,
                    i,
                    n;

                if (arguments.length > 1 || $.type(objects) === 'array') {
                    args = arguments.length > 1 ? arguments : objects;

                    for (i = 0, n = args.length; i < n; i += 1) {
                        self.add(args[i]);
                    }
                } else {
                    for (key in objects) {
                        if (Object.prototype.hasOwnProperty.call(objects, key)) {
                            self.add(key, objects[key]);
                        }
                    }
                }
            },

            insert: function (index, key, object) {
                if (arguments.length === 2) {
                    object = arguments[1];
                    key = self.getKey(object);
                }

                if (self.containsKey(key)){
                    self.removeKey(key);
                }

                if (index >= self.length){
                    return self.add(key, object);
                }

                self.length += 1;
                self.items.splice(index, 0, object);

                if (typeof key != 'undefined' && key !== null){
                    self.map[key] = object;
                }

                self.keys.splice(index, 0, key);

                return object;
            },

            replace: function(key, object){
                var old,
                    index;

                if (arguments.length === 1) {
                    object = arguments[0];
                    key = self.getKey(object);
                }

                old = self.map[key];

                if (typeof key == 'undefined' || key === null || typeof old == 'undefined') {
                    return self.add(key, object);
                }

                index = self.indexOfKey(key);

                self.items[index] = object;
                self.map[key] = object;

                return object;
            },

            /**
             * Returns the item associated with the passed key OR index.
             * Key has priority over index.
             * @param {String | Number} key
             * @return {Object | undefined}
             */
            getItem: function (key) {
                var object = self.map[key];

                return typeof object !== 'undefined' ? object : self.items[key];
            },

            getAt: function (index) {
                return self.items[index];
            },

            getKey: function (object) {
                return object.idProperty || object.id;
            },

            getKeyAt: function (index) {
                return self.keys[index];
            },

            indexOf: function (object) {
                return self.items.indexOf(object);
            },

            indexOfKey: function (key) {
                return self.keys.indexOf(key);
            },

            containsKey: function (key) {
                return typeof self.map[key] != 'undefined';
            },

            removeAt: function (index) {
                var object,
                    key;

                if (index < self.length && index >= 0) {
                    self.length--;
                    object = self.items[index];
                    self.items.splice(index, 1);
                    key = self.keys[index];

                    if (typeof key != 'undefined') {
                        delete self.map[key];
                    }

                    self.keys.splice(index, 1);

                    return object;
                }

                return false;
            },

            removeKey: function (key) {
                return self.removeAt(self.indexOfKey(key));
            },

            remove: function (object) {
                return self.removeAt(self.indexOf(object));
            },

            first: function () {
                return self.items[0];
            },

            last: function () {
                return self.items[self.length - 1];
            },
        };

        if (values) {
            self.addAll(values);
        }

        return self;
    };
})(jQuery);