(function ($) {
    $.jelastic = $.jelastic || {};
    $.jelastic.polling = $.jelastic.polling || {};

    /**
     * @param {$.jelastic.polling.ActionPoller} actionPoller
     */
    $.jelastic.polling.ActionHandler = function (actionPoller) {
        var self;

        self = {
            handlers: {},

            /**
             * @param {String} name
             * @param {Object} handler
             */
            add: function (name, handler) {
                if (self.handlers[name]) {
                    throw new Error("action handler [" + name + "] already exists!");
                }

                self.handlers[name] = handler;
            },

            /**
             * @private
             * @param {Object} handler
             * @param {Object} data
             * @return {Object | Array}
             */
            prepareParams: function (handler, data) {
                var resultParams,
                    receivedParams,
                    i,
                    n;

                receivedParams = (data.input || []).concat(data.output);

                if ($.type(handler.params) === "array") {
                    resultParams = {};

                    for (i = 0, n = handler.params.length; i < n; i += 1) {
                        resultParams[handler.params[i]] = receivedParams[i] || "";
                    }
                }

                return resultParams || receivedParams;
            },

            /**
             * @private
             * @param {String} eventName
             * @param {Object} data
             * @param {Object} trackedAction
             * @param {Function} [callback]
             */
            handleEvent: function (eventName, data, trackedAction, callback) {
                var handler = self.handlers[data.name],
                    params = self.prepareParams(handler, data);

                if (handler && $.type(handler[eventName]) === "function") {
                    handler[eventName].apply(handler, [data.id].concat(params).concat(trackedAction));
                }

                if ($.type(callback) === "function") {
                    callback.call(self, handler, params);
                }

                return params;
            },

            /**
             * @private
             * @param {Object} data
             * @param {Object} trackedAction
             */
            onCompositeActionStarted: function (data, trackedAction) {
                self.handleEvent("onStart", data, trackedAction);
            },

            /**
             * @private
             * @param {Object} data
             * @param {Object} trackedAction
             */
            onInternalActionCompleted: function (data, trackedAction) {
                self.handleEvent("onProgress", data, trackedAction, function (handler, params) {
                    if (params && params.eventName && $.type(handler.events) === "array") {
                        if (params.eventName === handler.events[handler.events.length - 1]) {
                            self.handleEvent("onSuccess", data, trackedAction);
                        }
                    }
                });
            },

            /**
             * @private
             * @param {Object} data
             * @param {Object} trackedAction
             */
            onInternalActionError: function (data, trackedAction) {
                self.handleEvent("onError", data, trackedAction);
            }
        };

        actionPoller.on("composite-action-started", self.onCompositeActionStarted, self);
        actionPoller.on("internal-action-completed", self.onInternalActionCompleted, self);
        actionPoller.on("internal-action-error", self.onInternalActionError, self);

        return self;
    };
})(jQuery);